/***************************************************************************
                          map.h  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Ralf-Christian Juergensen
    email                : ralf-christian.juergensen@stud.fh-flensburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MAP_H
#define MAP_H

#include "field.h"
#include "globals.h"

#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <string.h>
#include <ctype.h>

/** The map to play on.
  * Maps contain several fields. The definition of a map can be loaded from an ASCII-file.
  * See file map.readme for more details.
  */
class Map
{
public:
    /** Constructs a new map and load a map-file */
    Map(const char* mapFileName);
    /** Copy an entire map from another map*/
    Map(Map& map);
    /** Destroy the map */
    ~Map(void);
    
    /** Load a new map */
    void load(const char* mapFileName);
    		
    void and(int x, int y, bool bit);		
    void or(int x, int y, bool bit);		
    void xor(int x, int y, bool bit);
    		
    /** Returns number of fields with matching player and value */
    int checkFields(Player player, Value value);		
    
    Value getFieldValue(int x, int y);
    Player getFieldPlayer(int x, int y);
    		
    int getWidth(void);
    int getHeight(void);
    		
private:
    int width;      // width in fields
    int height;     // height in fields
    Field* fields;  // array of fields

    void deleteFields(void);
    		
    /** Calculates linear position in field-array */
    inline int coords(int x, int y)
    {
        return x + y * width;
    }
    
    class Token
    {
    private:
        void delValue() {if(value != 0) delete value;}
        void delTokenStr() {if(tokenStr != 0) delete[] tokenStr;}
    public:
        Token() {value = 0; tokenStr = 0; isMust = false;}
        ~Token()
        {
            delValue();
            delTokenStr();
        }
        void setToken(const char* str, bool must)
        {
            delTokenStr();
            tokenStr = new char[strlen(str)+1];
            strcpy(tokenStr, str);
            isMust = must;
        }
        void setValue(void* val)
        {
            delValue();
            value = val;
        }
        void* getValue() {return value;}
        char* getTokenStr() {return tokenStr;}
        bool isValid() {return ((value != 0) || !isMust);}
    private:
        char* tokenStr;
        void* value;
        bool isMust;
    };
    enum TokenID
    {
        WIDTH=0,
        HEIGHT,
        PLAYER_ONE,
        PLAYER_TWO,
        NO_PLAYER,
        ON_FIELD,
        OFF_FIELD,
        EMPTY_FIELD,
        PENGUIN_FIELD,
        LAST_TOKEN      // This is not a valid token and MUST be the last in this list!
    };
    
    void parseDefines(ifstream& mapFile, Token* tokenList);
    /** Read an integer-value from string with following syntax:
      * "TOKEN=INT;"
      */
    int* readInt(const char* string, const char* token);
    /** Read a char from string with following syntax:
      * "TOKEN=CHAR;"
      */
    char* readChar(const char* string, const char* token);
};

#endif // MAP_H
