/***************************************************************************
                          playground.cpp  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Ralf-Christian Juergensen
    email                : ralf-christian.juergensen@stud.fh-flensburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/******************************************************************************
	class PlayGround

	The core of the game. Consists the game logic.
*******************************************************************************/

#include "playground.h"

PlayGround::PlayGround(Player startPlayer, const char* mapFileName) : map(mapFileName)
{
    player = startPlayer;
}

void PlayGround::newGame(Player p, const char* mapFileName)
{
    player = p;
    	
    map.load(mapFileName);
    
    cursor.set();	
    initRound();
}

void PlayGround::initRound(void)
{
    cursor.init();
}

void PlayGround::setCursor(void)
{
    int x = cursor.getXPos();   // current x position of the cursor
    int y = cursor.getYPos();   // current y position of the cursor
    Type t = cursor.getType();  // current type of the cursor
            
    // if type is '?' generate 'real' type
    if(t == RND)
    {
        t = Type(3.0 * rand() / (RAND_MAX + 1.0));
    }
            
    try
    {
        switch(t)
        {
        case AND:
            map.and(x,   y,   cursor.getBit(0));
            map.and(x+1, y,   cursor.getBit(1));
            map.and(x+2, y,   cursor.getBit(2));
            map.and(x,   y+1, cursor.getBit(3));
            map.and(x+2, y+1, cursor.getBit(4));
            map.and(x,   y+2, cursor.getBit(5));
            map.and(x+1, y+2, cursor.getBit(6));
            map.and(x+2, y+2, cursor.getBit(7));
            break;
        case OR:
            map.or (x,   y,   cursor.getBit(0));
            map.or (x+1, y,   cursor.getBit(1));
            map.or (x+2, y,   cursor.getBit(2));
            map.or (x,   y+1, cursor.getBit(3));
            map.or (x+2, y+1, cursor.getBit(4));
            map.or (x,   y+2, cursor.getBit(5));
            map.or (x+1, y+2, cursor.getBit(6));
            map.or (x+2, y+2, cursor.getBit(7));
            break;
        case XOR:
            map.xor(x,   y,   cursor.getBit(0));
            map.xor(x+1, y,   cursor.getBit(1));
            map.xor(x+2, y,   cursor.getBit(2));
            map.xor(x,   y+1, cursor.getBit(3));
            map.xor(x+2, y+1, cursor.getBit(4));
            map.xor(x,   y+2, cursor.getBit(5));
            map.xor(x+1, y+2, cursor.getBit(6));
            map.xor(x+2, y+2, cursor.getBit(7));
            break;
        case RND:
            break;  // do nothing (prevent compiler warning)
        }
    }
    catch (char* errorMsg)
    {
        cerr << "error!\n";
        cerr << errorMsg << '\n';
        cerr << "called by PlayGround::setCursor().\n";
        exit(-1);
    }
}

void PlayGround::cursorUp(void)
{
    if(cursor.getYPos() > 0)
    {
        cursor.goUp();
    }
}

void PlayGround::cursorDown(void)
{
    // attention: the cursor is 3 fields high
    if(cursor.getYPos() < (map.getHeight() - 3))
    {
        cursor.goDown();
    }
}

void PlayGround::cursorLeft(void)
{
    if(cursor.getXPos() > 0)
    {
        cursor.goLeft();
    }
}

void PlayGround::cursorRight(void)
{
    // attention: the cursor is 3 fields wide
    if(cursor.getXPos() < (map.getWidth() - 3))
    {
        cursor.goRight();
    }
}

int PlayGround::getCursorX(void)
{
    return cursor.getXPos();
}

int PlayGround::getCursorY(void)
{
    return cursor.getYPos();
}

Player PlayGround::checkWinner(void)
{
    // get number of fields for player one that are 0(OFF)
    int offOne = getScore(ONE);
    int offTwo = getScore(TWO);
    
    
    // when both players are winner => winner is current player
    if((offOne == 0) && (offTwo == 0))
    {
        return player;
    }
    	
    if(offOne == 0)
    {
        return ONE;
    }
    	
    if(offTwo == 0)
    {
        return TWO;
    }
    	
    return NONE;	
}

void PlayGround::changePlayer(void)
{
    if(player == ONE)
    {
        player = TWO;
    }
    else
    {
        player = ONE;
    }
}

int PlayGround::getWidth(void)
{
    return map.getWidth();
}

int PlayGround::getHeight(void)
{
    return map.getHeight();
}

Type PlayGround::getCursorType(void)
{
    return cursor.getType();
}

bool PlayGround::getCursorBit(int no)
{
    return cursor.getBit(no);
}

Value PlayGround::getFieldValue(int x, int y)
{
    return map.getFieldValue(x, y);
}

Player PlayGround::getFieldPlayer(int x, int y)
{
    return map.getFieldPlayer(x, y);
}

Player PlayGround::getPlayer(void)
{
    return player;
}

int PlayGround::getScore(Player p)
{
    return map.checkFields(p, OFF);
}
